# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/genomenotebook',
                'doc_host': 'https://dbikard.github.io',
                'git_url': 'https://github.com/dbikard/genomenotebook',
                'lib_path': 'genomenotebook'},
  'syms': { 'genomenotebook.browser': { 'genomenotebook.browser.GenomeBrowser': ('browser.html#genomebrowser', 'genomenotebook/browser.py'),
                                        'genomenotebook.browser.GenomeBrowser.__init__': ( 'browser.html#genomebrowser.__init__',
                                                                                           'genomenotebook/browser.py'),
                                        'genomenotebook.browser.GenomeBrowser.add_track': ( 'browser.html#genomebrowser.add_track',
                                                                                            'genomenotebook/browser.py'),
                                        'genomenotebook.browser.GenomeBrowser.get_browser': ( 'browser.html#genomebrowser.get_browser',
                                                                                              'genomenotebook/browser.py'),
                                        'genomenotebook.browser.GenomeBrowser.get_search_box': ( 'browser.html#genomebrowser.get_search_box',
                                                                                                 'genomenotebook/browser.py'),
                                        'genomenotebook.browser.GenomeBrowser.show': ( 'browser.html#genomebrowser.show',
                                                                                       'genomenotebook/browser.py'),
                                        'genomenotebook.browser.Track': ('browser.html#track', 'genomenotebook/browser.py'),
                                        'genomenotebook.browser.Track.__init__': ( 'browser.html#track.__init__',
                                                                                   'genomenotebook/browser.py'),
                                        'genomenotebook.browser.Track.bar': ('browser.html#track.bar', 'genomenotebook/browser.py'),
                                        'genomenotebook.browser.Track.line': ('browser.html#track.line', 'genomenotebook/browser.py'),
                                        'genomenotebook.browser.Track.scatter': ( 'browser.html#track.scatter',
                                                                                  'genomenotebook/browser.py')},
            'genomenotebook.examples': {},
            'genomenotebook.js_callback_code': {},
            'genomenotebook.plotting': { 'genomenotebook.plotting.arrow_patch': ('plotting.html#arrow_patch', 'genomenotebook/plotting.py'),
                                         'genomenotebook.plotting.create_genome_browser_plot': ( 'plotting.html#create_genome_browser_plot',
                                                                                                 'genomenotebook/plotting.py'),
                                         'genomenotebook.plotting.get_all_glyphs': ( 'plotting.html#get_all_glyphs',
                                                                                     'genomenotebook/plotting.py'),
                                         'genomenotebook.plotting.get_arrow_patch': ( 'plotting.html#get_arrow_patch',
                                                                                      'genomenotebook/plotting.py'),
                                         'genomenotebook.plotting.get_gene_patches': ( 'plotting.html#get_gene_patches',
                                                                                       'genomenotebook/plotting.py'),
                                         'genomenotebook.plotting.get_genes_from_annotation': ( 'plotting.html#get_genes_from_annotation',
                                                                                                'genomenotebook/plotting.py'),
                                         'genomenotebook.plotting.get_genome_annotations': ( 'plotting.html#get_genome_annotations',
                                                                                             'genomenotebook/plotting.py'),
                                         'genomenotebook.plotting.get_y_range': ('plotting.html#get_y_range', 'genomenotebook/plotting.py'),
                                         'genomenotebook.plotting.rect_patch': ('plotting.html#rect_patch', 'genomenotebook/plotting.py')}}}
